
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "avej_util_tile_map.h"

CTileMap::CTileMap(void)
: m_pMapData(NULL)
{
	memset(&m_mapHeader, 0, sizeof(m_mapHeader));
	memset(&m_tileInfo, 0, sizeof(m_tileInfo));
}

CTileMap::~CTileMap(void)
{
	if (m_pMapData)
	{
		delete[] m_pMapData;
		m_pMapData = NULL;
	}
}

void  CTileMap::m_SetMoveableTile(int attribute, int ixTile, bool isMoveable)
{
	if (isMoveable)
	{
		m_mapHeader.moveableTable[attribute][ixTile / 8] &= (0xFF ^ (1 << (ixTile % 8)));
	}
	else
	{
		m_mapHeader.moveableTable[attribute][ixTile / 8] |= (1 << (ixTile % 8));
	}
}

void  CTileMap::m_SetPassableTile(int attribute, int ixTile, bool isPassable)
{
	if (isPassable)
	{
		m_mapHeader.passableTable[attribute][ixTile / 8] &= (0xFF ^ (1 << (ixTile % 8)));
	}
	else
	{
		m_mapHeader.passableTable[attribute][ixTile / 8] |= (1 << (ixTile % 8));
	}
}

TMapData* CTileMap::Map(int x, int y)
{
	static TMapData dummy;

	if ((x < 0) || (x >= m_mapHeader.wSize) || (y < 0) || (y >= m_mapHeader.hSize))
	{
		dummy.packedData = 0;
		dummy.data.tile = m_mapHeader.defaultTile;
		return &dummy;
	}

	return (m_pMapData + y * m_mapHeader.pitch + x);
}

int* CTileMap::UserParameter(int index)
{
	static int dummyArray[MAX_USER_PARAMETER] = {0, };

	// out of range
	if ((index < 0) || (index >= MAX_USER_INDEX))
	{
		memset(dummyArray, 0, sizeof(dummyArray));
		return dummyArray;
	}

	return m_userParameter[index];
}

bool CTileMap::IsMoveableMap(int x, int y)
{
	if ((x < 0) || (x >= m_mapHeader.wSize) || (y < 0) || (y >= m_mapHeader.hSize))
		return false;

	TMapData* mapData = (m_pMapData + y * m_mapHeader.pitch + x);

	if ((m_mapHeader.moveableTable[0][mapData->data.tile / 8] & (1 << (mapData->data.tile % 8))))
		return false;

	if ((m_mapHeader.moveableTable[1][mapData->data.object / 8] & (1 << (mapData->data.object % 8))))
		return false;

	if (mapData->data.character)
		return false;

	return true;
}

bool CTileMap::IsPassableMap(int x, int y)
{
	if ((x < 0) || (x >= m_mapHeader.wSize) || (y < 0) || (y >= m_mapHeader.hSize))
		return false;

	TMapData* mapData = (m_pMapData + y * m_mapHeader.pitch + x);

//	if (mapData->data.object == 16)
//		return true;

	if ((m_mapHeader.passableTable[0][mapData->data.tile / 8] & (1 << (mapData->data.tile % 8))) > 0)
		return false;

	return true;
}

bool CTileMap::Create(const char* szId, int width, int heigth)
{
	this->Destroy();

	strncpy(m_mapHeader.id, szId, sizeof(m_mapHeader.id));
	m_mapHeader.id[sizeof(m_mapHeader.id)-1] = 0;

	m_mapHeader.wSize = width;
	m_mapHeader.hSize = heigth;
	m_mapHeader.pitch = m_mapHeader.wSize;
	m_mapHeader.enablePlane = 0;

	m_pMapData = new TMapData[m_mapHeader.pitch * m_mapHeader.hSize];
	if (m_pMapData == NULL)
		return false;

	memset(m_pMapData, 0, m_mapHeader.pitch * m_mapHeader.hSize * sizeof(TMapData));

	return true;
}

bool CTileMap::Destroy(void)
{
	if (m_pMapData)
	{
		delete[] m_pMapData;
		m_pMapData = NULL;
	}

	memset(&m_mapHeader, 0, sizeof(m_mapHeader));

	return true;
}

bool CTileMap::LoadFromText(const char* szTextFile)
{
	FILE* pfText;
	char  szLine[1024];
	char* pLine;
	char  szCommand[32];
	char* pData;

	pfText = fopen(szTextFile, "rt");
	if (pfText == NULL)
		return false;

	memset(&m_mapHeader, 0, sizeof(m_mapHeader));

	// passable map 
	memset(m_mapHeader.passableTable, 0xFF, sizeof(m_mapHeader.passableTable));
	m_SetPassableTile(0, 0, true);
	m_SetPassableTile(0, 1, true);

	// moveable map 
	memset(m_mapHeader.moveableTable, 0xFF, sizeof(m_mapHeader.moveableTable));
	m_SetMoveableTile(0, 0, true);
	m_SetMoveableTile(0, 1, true);

	// 'def MAP_HEADER' ڿ ִ  ˻
	while (!feof(pfText))
	{
		fgets(szLine, 1024, pfText);
		szLine[strlen(szLine)-1] = 0;
		if (strcmp(szLine, "def MAP_HEADER") == 0)
			break;
	}

	// map header  Ľ
	while (!feof(pfText))
	{
		fgets(szLine, 1024, pfText);
		szLine[strlen(szLine)-1] = 0;

		if ((szLine[0] == 0) || (szLine[0] == 0x0A) || (szLine[0] == '#'))
			continue;

		if (szLine[0] != 0x09)
		{
			if (strcmp(szLine, "def MAP_DATA") == 0)
				break;
		}
		else
		{
			// 'command: data' 
			pLine = szLine + 1;
			pData = strstr(pLine, ":");
			if (pData)
			{
				// command κи ߶ 
				strncpy(szCommand, pLine, pData - pLine);
				szCommand[pData - pLine] = 0;
				++pData;

				// data κ  
				while (*pData)
				{
					if (*pData != ' ')
						break;
					++pData;
				}

				if (strcmp(szCommand, "id") == 0)
				{
					strncpy(m_mapHeader.id, pData, sizeof(m_mapHeader.id));
					continue;
				}
				if (strcmp(szCommand, "wSize") == 0)
				{
					m_mapHeader.wSize = atoi(pData);
					m_mapHeader.pitch = m_mapHeader.wSize;
					continue;
				}
				if (strcmp(szCommand, "hSize") == 0)
				{
					m_mapHeader.hSize = atoi(pData);
					continue;
				}
				if (strcmp(szCommand, "default") == 0)
				{
					m_mapHeader.defaultTile = atoi(pData);
					continue;
				}
			
			}
		}
	}

	m_pMapData = new TMapData[m_mapHeader.pitch * m_mapHeader.hSize];
	memset(m_pMapData, 0, m_mapHeader.pitch * m_mapHeader.hSize * sizeof(TMapData));

	//  б
	for (int y = 0; y < m_mapHeader.hSize; y++)
	{
		if (feof(pfText))
			break;

		fgets(szLine, 1024, pfText);

		for (int x = 0; x < m_mapHeader.wSize; x++)
		{
			if (szLine[x] == ' ')
				this->Map(x, y)->data.tile = 1;
			else if ((szLine[x] >= '@') && (szLine[x] <= 'Z'))
				this->Map(x, y)->data.character = szLine[x] - '@' + 1;
			else
				this->Map(x, y)->data.tile = 20;
		}
	}

	// 'def CHARA_DATA' ڿ ִ  ˻
	while (!feof(pfText))
	{
		fgets(szLine, 1024, pfText);
		szLine[strlen(szLine)-1] = 0;
		if (strcmp(szLine, "def CHARA_DATA") == 0)
			break;
	}

	// character data Ľ
	while (!feof(pfText))
	{
		fgets(szLine, 1024, pfText);
		szLine[strlen(szLine)-1] = 0;

		if ((szLine[0] == 0) || (szLine[0] == 0x0A) || (szLine[0] == '#'))
			continue;

		if (szLine[0] != 0x09)
		{
			if (strcmp(szLine, "def MAP_DATA") == 0)
				break;
		}
		else
		{
			char delimiter[2];
			char comma[MAX_USER_PARAMETER-1];
			char ixChara; // from '@' to 'Z'
			int  data[MAX_USER_PARAMETER]; // id, face, face dir, type
			int  offset;

			memset(data, 0, sizeof(data));
			offset = sscanf
				(
					szLine, "%c %c %c %d %c %d %c %d %c %d %c %d %c %d",
					&delimiter[0], &ixChara, &delimiter[1],
					&data[0], &comma[0], &data[1], &comma[1], &data[2], &comma[2], &data[3], &comma[3], &data[4], &comma[4],
					&data[5]//, &comma[5], &data[6], &comma[6], &data[7], &comma[7], &data[8], &comma[8], &data[9]
				);
			// Revise the number of actual parameters
			offset = ((offset - 3) + 1) / 2;

			memcpy(m_userParameter[ixChara - '@' + 1], data, sizeof(data));
		}
	}

	fclose(pfText);

	return true;
}

#define CHECK_VALID(a) if (!(a)) throw 0;

bool CTileMap::LoadFromFile(const char* szMapName)
{
	assert(m_pMapData == NULL);

	FILE*  pfIn = NULL;
	size_t readBytes;

	try
	{
		pfIn = fopen(szMapName, "rb");
		CHECK_VALID(pfIn != NULL);

		readBytes = fread(&m_mapHeader, sizeof(m_mapHeader), 1, pfIn);
		CHECK_VALID(readBytes == 1);

		m_pMapData = new TMapData[m_mapHeader.pitch * m_mapHeader.hSize];
		CHECK_VALID(m_pMapData != NULL);

		readBytes = fread(m_pMapData, m_mapHeader.pitch * m_mapHeader.hSize, 1, pfIn);
		CHECK_VALID(readBytes == 1);
	}
	catch (...)
	{
		delete[] m_pMapData;
		m_pMapData = NULL;

		if (pfIn)
			fclose(pfIn);

		return false;
	}

	fclose(pfIn);

	return true;
}

bool CTileMap::SaveToFile(const char* szMapName)
{
	FILE* pfOut;

	pfOut = fopen(szMapName, "wb");

	if (pfOut == NULL)
		return false;

	fwrite(&m_mapHeader, sizeof(m_mapHeader), 1, pfOut);
	fwrite(m_pMapData, m_mapHeader.pitch * m_mapHeader.hSize, 1, pfOut);

	fclose(pfOut);

	return true;
}

bool CTileMap::LoadFromFile_Lore(const char* szMapName)
{
	assert(m_pMapData == NULL);

	#pragma pack(push, 1)
	struct TMapHeader // sizeof = 96
	{
		char ID[11];
		unsigned char xmax;
		unsigned char ymax;
		unsigned char Tile_Type;
		unsigned char encounter;
		unsigned char handicap;
		unsigned char startX;
		unsigned char startY;
		char exitmap[11];
		unsigned char exitX;
		unsigned char exitY;
		char entermap[11];
		unsigned char enterX;
		unsigned char enterY;
		unsigned char _default;
		unsigned char handicap_bit;
		char unused[50];
	};
	#pragma pack(pop)

	TMapHeader mapHeader;
	unsigned char* pMapData = NULL;

	FILE* pfIn = NULL;
	m_pMapData = NULL;

	try
	{
		size_t readBytes;

		{
			pfIn = fopen("./LORE.M", "rb");
			CHECK_VALID(pfIn != NULL);

			readBytes = fread(&mapHeader, sizeof(mapHeader), 1, pfIn);
			CHECK_VALID(readBytes == 1);

			pMapData  = new unsigned char[mapHeader.xmax * mapHeader.ymax];
			CHECK_VALID(pMapData != NULL);

			readBytes = fread(pMapData, mapHeader.xmax * mapHeader.ymax, 1, pfIn);
			CHECK_VALID(readBytes == 1);

			fclose(pfIn);
			pfIn = NULL;
		}

		memset(&m_mapHeader, 0, sizeof(m_mapHeader));

		memcpy(m_mapHeader.id, mapHeader.ID+1, mapHeader.ID[0]);
		m_mapHeader.wSize = mapHeader.xmax;
		m_mapHeader.hSize = mapHeader.ymax;
		m_mapHeader.pitch = mapHeader.xmax;
//		m_mapHeader.moveableTable[4][256/8];
//		m_mapHeader.passableTable[4][256/8];
		m_mapHeader.enablePlane = 0;
		m_mapHeader.defaultTile = 0;

		int i;
		for (i = 1; i <= 18; i++)
			m_SetMoveableTile(0, i, false);
		for (i = 20; i <= 21; i++)
			m_SetMoveableTile(0, i, false);
		for (i = 5; i <= 11; i++)
			m_SetPassableTile(0, i, false);
		for (i = 16; i <= 16; i++)
			m_SetPassableTile(0, i, false);

		m_pMapData = new TMapData[m_mapHeader.pitch * m_mapHeader.hSize];
		CHECK_VALID(m_pMapData != NULL);

		memset(m_pMapData, 0, sizeof(TMapData) * (m_mapHeader.pitch * m_mapHeader.hSize));

		for (int y = 0; y < m_mapHeader.hSize; y++)
		for (int x = 0; x < m_mapHeader.wSize; x++)
		{
			m_pMapData[y * m_mapHeader.pitch + x].data.tile = pMapData[y * mapHeader.xmax + x] & 0x7F;
		}

		// register tile information
		for (i = 0; i < 60; i++)
		{
			switch (rand() % 5)
			{
			case 0:
				m_tileInfo[0][i].szName = "ü";
				break;
			case 1:
				m_tileInfo[0][i].szName = "";
				break;
			case 2:
				m_tileInfo[0][i].szName = "Ʈ";
				break;
			case 3:
				m_tileInfo[0][i].szName = " ";
				break;
			case 4:
				m_tileInfo[0][i].szName = "̻ ";
				break;
			}
		}

		m_pMapData[mapHeader.startY * m_mapHeader.pitch + mapHeader.startX].data.character = 1;
		memset(m_userParameter[1], 0, sizeof(m_userParameter[1]));

		m_pMapData[93 * m_mapHeader.pitch + 51].data.character = 2;
		memset(m_userParameter[2], 0, sizeof(m_userParameter[2]));
		m_userParameter[2][1] = 1;

		m_pMapData[53 * m_mapHeader.pitch + 50].data.character = 3;
		memset(m_userParameter[3], 0, sizeof(m_userParameter[3]));
		m_userParameter[3][1] = 2;

		m_pMapData[54 * m_mapHeader.pitch + 51].data.character = 4;
		memset(m_userParameter[4], 0, sizeof(m_userParameter[4]));
		m_userParameter[4][1] = 3;

		delete[] pMapData;

		return true;
	}
	catch (...)
	{
		delete[] m_pMapData;
		m_pMapData = NULL;

		delete[] pMapData;
		pMapData = NULL;

		if (pfIn)
			fclose(pfIn);

		return false;
	}
}

////////////////////////////////////////////////////////////////////////////////
//?? ̰͵ ġ Ⱑ ƴϴ.

const long OZSEEK_SET = 0;
const long OZSEEK_CUR = 1;
const long OZSEEK_END = 2;

class __CReadStream
{ 
public:
	virtual ~__CReadStream(void) { ; };
	int isAvailable;
		
	virtual long  Read(void* pBuffer, long Count) = 0;
	virtual long  Seek(long Offset, unsigned short Origin) = 0;
	virtual long  GetSize(void) = 0;
	virtual void* GetPointer(void) = 0;
	virtual bool  IsValidPos(void) = 0;
};


#if !defined(_TARGET_BADA)

class __CFileReadStream : public __CReadStream
{
public:
	__CFileReadStream(const char* szFileName);
	~__CFileReadStream(void);

private:
	FILE* m_pFile;

public:
	long  Read(void* pBuffer, long Count);
	long  Seek(long Offset, unsigned short Origin);
	long  GetSize(void);
	void* GetPointer(void) { return NULL; };
	bool  IsValidPos(void);
};

__CFileReadStream::__CFileReadStream(const char* szFileName)
{
	m_pFile = fopen(szFileName, "rb");

	this->isAvailable = (m_pFile != 0);
}

__CFileReadStream::~__CFileReadStream(void)
{
	if (this->isAvailable)
		fclose(m_pFile);
}

long __CFileReadStream::Read(void* pBuffer, long Count)
{
	if (!this->isAvailable)
		return 0;

	return fread(pBuffer, 1, Count, m_pFile);
}

long __CFileReadStream::Seek(long Offset, unsigned short Origin)
{
	if (!this->isAvailable)
		return -1;

	switch (Origin)
	{
	case OZSEEK_SET:
	case OZSEEK_CUR:
	case OZSEEK_END:
		fseek(m_pFile, Offset, Origin);
		break;
	default:
		return -1;
	}

	return ftell(m_pFile);
}

long __CFileReadStream::GetSize(void)
{
	if (!this->isAvailable)
		return -1;

	long  Result;
	long  CurrentPos = ftell(m_pFile);

	fseek(m_pFile, 0, SEEK_END);
	Result = ftell(m_pFile);

	fseek(m_pFile, CurrentPos, SEEK_SET);

	return Result;
}

bool __CFileReadStream::IsValidPos(void)
{
	if (!this->isAvailable)
		return false;

	return (feof(m_pFile) == 0);
}

#else

#include <FIo.h>
const long OZSEEK_SET = Osp::Io::FILESEEKPOSITION_BEGIN;
const long OZSEEK_CUR = Osp::Io::FILESEEKPOSITION_CURRENT;
const long OZSEEK_END = Osp::Io::FILESEEKPOSITION_END;

class __CFileReadStream : public __CReadStream
{
public:
	__CFileReadStream(const char* szFileName);
	~__CFileReadStream(void);

private:
	Osp::Io::File* m_pFile;
	int m_file_size;

public:
	long  Read(void* pBuffer, long Count);
	long  Seek(long Offset, unsigned short Origin);
	long  GetSize(void);
	void* GetPointer(void) { return NULL; };
	bool  IsValidPos(void);
};

__CFileReadStream::__CFileReadStream(const char* szFileName)
{
	result r;
	Osp::Base::String fileName(szFileName);

	fileName = "/Res/" + fileName;

	m_pFile = new Osp::Io::File;

	r = m_pFile->Construct(fileName, L"r");

	if (r == E_SUCCESS)
	{
		m_file_size = this->GetSize();
	}
	else
	{
		AppLog("[SMGAL] File not found");
		delete m_pFile;
		m_pFile = 0;
	}

	this->isAvailable = (m_pFile != 0);
}

__CFileReadStream::~__CFileReadStream(void)
{
	if (this->isAvailable)
		delete m_pFile;
}

long __CFileReadStream::Read(void* pBuffer, long Count)
{
	if (!this->isAvailable)
		return 0;

	return m_pFile->Read(pBuffer, Count);
}

long __CFileReadStream::Seek(long Offset, unsigned short Origin)
{
	if (!this->isAvailable)
		return -1;

	switch (Origin)
	{
	case OZSEEK_SET:
	case OZSEEK_CUR:
	case OZSEEK_END:
		m_pFile->Seek(Osp::Io::FileSeekPosition(Origin), Offset);
		break;
	default:
		return -1;
	}

	return m_pFile->Tell();
}

long __CFileReadStream::GetSize(void)
{
	if (!this->isAvailable)
		return -1;

	long  Result;
	long  CurrentPos = m_pFile->Tell();

	this->Seek(0, OZSEEK_END);
	Result = m_pFile->Tell();

	this->Seek(CurrentPos, OZSEEK_SET);

	return Result;
}

bool __CFileReadStream::IsValidPos(void)
{
	if (!this->isAvailable)
		return false;

	return (m_pFile->Tell() < m_file_size);
}

#endif



////////////////////////////////////////////////////////////////////////////////

unsigned char SET_MOVEABLE_OBJECT[]    = {0xAF,0x0A,0x00,0x81,0xFF,0x3F,0x08,0x00,0x00,0x00,0x00,0x00,0xF0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};

bool CTileMap::LoadFromFile_Dejavu(const char* szMapName)
{
	__CReadStream* pStream = new __CFileReadStream(szMapName);

	//__try
	{
		if (!pStream->isAvailable)
			return false;

		char  mapID[4];
		short mapW, mapH;

		pStream->Read(mapID, 4);
		pStream->Read(&mapW, 2);
		pStream->Read(&mapH, 2);

		memset(&m_mapHeader, 0, sizeof(m_mapHeader));

		m_mapHeader.wSize = mapW;
		m_mapHeader.hSize = mapH;
		m_mapHeader.pitch = mapW;
		m_mapHeader.enablePlane = 0;
		m_mapHeader.defaultTile = 0;

		int i, j;

		for (i = 0; i <= 255; i++) 
		{
			m_SetMoveableTile(0, i, false);
			m_SetPassableTile(0, i, false);
		}
		for (i = 1; i <= 89; i++) 
		{
			m_SetMoveableTile(0, i, true);
			m_SetPassableTile(0, i, true);
		}
		for (i = 90; i <= 95; i++) 
		{
			m_SetPassableTile(0, i, true);
		}

		m_SetMoveableTile(0, 140, true);
		m_SetPassableTile(0, 140, true);
		m_SetMoveableTile(0, 143, true);
		m_SetPassableTile(0, 143, true);

		m_SetMoveableTile(0, 100, true);
		m_SetMoveableTile(0, 102, true);
		m_SetMoveableTile(0, 104, true);
//		m_SetMoveableTile(0, 105, true);
		m_SetMoveableTile(0, 106, true);
//		m_SetMoveableTile(0, 107, true);
		m_SetMoveableTile(0, 108, true);
		m_SetMoveableTile(0, 110, true);
		m_SetMoveableTile(0, 112, true);
		m_SetMoveableTile(0, 114, true);
		m_SetMoveableTile(0, 118, true);
		m_SetMoveableTile(0, 120, true);
		m_SetMoveableTile(0, 122, true);
		m_SetMoveableTile(0, 124, true);
//		m_SetMoveableTile(0, 125, true);
		m_SetMoveableTile(0, 126, true);
//		m_SetMoveableTile(0, 127, true);
		m_SetMoveableTile(0, 128, true);
		m_SetMoveableTile(0, 130, true);
		m_SetMoveableTile(0, 132, true);
		m_SetMoveableTile(0, 134, true);
		m_SetMoveableTile(0, 138, true);

		m_SetPassableTile(0, 100, true);
		m_SetPassableTile(0, 102, true);
		m_SetPassableTile(0, 104, true);
//		m_SetPassableTile(0, 105, true);
		m_SetPassableTile(0, 106, true);
//		m_SetPassableTile(0, 107, true);
		m_SetPassableTile(0, 108, true);
		m_SetPassableTile(0, 110, true);
		m_SetPassableTile(0, 112, true);
		m_SetPassableTile(0, 114, true);
		m_SetPassableTile(0, 118, true);
		m_SetPassableTile(0, 120, true);
		m_SetPassableTile(0, 122, true);
		m_SetPassableTile(0, 124, true);
//		m_SetPassableTile(0, 125, true);
		m_SetPassableTile(0, 126, true);
//		m_SetPassableTile(0, 127, true);
		m_SetPassableTile(0, 128, true);
		m_SetPassableTile(0, 130, true);
		m_SetPassableTile(0, 132, true);
		m_SetPassableTile(0, 134, true);
		m_SetPassableTile(0, 138, true);

		memcpy(m_mapHeader.moveableTable[MAPDATA_OBJECT], SET_MOVEABLE_OBJECT, sizeof(m_mapHeader.moveableTable[MAPDATA_OBJECT]));
		for (i = 0; i < int(sizeof(m_mapHeader.moveableTable[MAPDATA_OBJECT])); i++)
			m_mapHeader.moveableTable[MAPDATA_OBJECT][i] ^= 0xFF;


		m_pMapData = new TMapData[m_mapHeader.pitch * m_mapHeader.hSize];
		if (m_pMapData == NULL)
			return false;

		TMapData map_data = {0};

		// read tile data
		for (j = 0; j < m_mapHeader.hSize; j++)
		for (i = 0; i < m_mapHeader.wSize; i++)
		{
			pStream->Read(&map_data.data.tile, 1);
			m_pMapData[j*m_mapHeader.pitch+i] = map_data;
		}

		// read object data
		for (j = 0; j < m_mapHeader.hSize; j++)
		for (i = 0; i < m_mapHeader.wSize; i++)
		{
			map_data = m_pMapData[j*m_mapHeader.pitch+i];
			pStream->Read(&map_data.data.object, 1);
			m_pMapData[j*m_mapHeader.pitch+i] = map_data;
		}

		m_pMapData[30 * m_mapHeader.pitch + 65].data.character = 1;
		memset(m_userParameter[1], 0, sizeof(m_userParameter[1]));

		m_pMapData[26 * m_mapHeader.pitch + 62].data.character = 2;
		memset(m_userParameter[2], 0, sizeof(m_userParameter[2]));

		m_pMapData[30 * m_mapHeader.pitch + 64+4].data.character = 3;
		memset(m_userParameter[3], 0, sizeof(m_userParameter[3]));

		m_pMapData[22 * m_mapHeader.pitch + 64].data.character = 4;
		memset(m_userParameter[3], 0, sizeof(m_userParameter[3]));
	}
	//__finally
	{
		delete pStream;

	}

	return true;
}

#if 0

unsigned char SET_FOREGROUND_IN_TOWN[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x50,0x5F,0x45,0xF5,0x55,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
unsigned char SET_FOREGROUND_IN_DEN[]  = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x75,0x55,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
unsigned char SET_OBJECT_TILE[]        = {0x20,0x08,0x00,0x01,0x00,0x3C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
unsigned char SET_MOVEABLE_OBJECT[]    = {0xAF,0x0A,0x00,0x81,0xFF,0x3F,0x08,0x00,0x00,0x00,0x00,0x00,0xF0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};
unsigned char SET_PASSABLE_OBJECT[]    = {0x87,0x02,0x00,0x80,0x00,0x3C,0x00,0x00,0x00,0x00,0x00,0x00,0xF0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,};
/*
   FOREGROUND_IN_TILE_IMAGE : set of byte =
      [100,102,104..108,110,112,114,118,120,122,124..128,130,132,134,138];
   FOREGROUND_IN_DEN_IMAGE  : set of byte =
      [120,122,124..126,128,130,132,134,138];
   FOREGROUND_OBJECT_TILE   : SByte = [5,11,24,42..45];
   MOVABLE_OBJECT_TILE      : SByte = [0..3,5,7,9,11,24,31..45,51,100..255];

{
   STAIRS_TILE            : SByte = [140..143];
   MOVABLE_TILE           : SByte = [1..89,140,143];

   MOVABLE_OBJECT_TILE    : SByte = [0..3,5,7,9,11,24,31..45,51,100..255];
   MOVABLE_FIELD_TILE     : SByte = [__FIELD_EMPTY,__FIELD_FIRE,__FIELD_VACUUM,__FIELD_POISON,__FIELD_SLEEP,__FIELD_ENERGY];
   PASSABLE_OBJECT_TILE   : SByte = [0..2,7,9,31,42..45,100..255];
   PASSABLE_FIELD_TILE    : SByte = [0..5];
}
*/

const
	SET_DATA = [100,102,104..108,110,112,114,118,120,122,124..128,130,132,134,138];

procedure TForm1.Button1Click(Sender: TObject);
var
	setData: array[0..pred(256 div 2)] of byte;
	i: integer;
	s: string;
	index: integer;
	mask: byte;
begin
	fillchar(setData, sizeof(setData), 0);

	for i := 0 to 255 do begin
		if i in SET_DATA then begin
			index := i div 8;
			mask  := 1 shl (i mod 8);
			setData[index] := setData[index] or mask;
		end;
	end;

	s := 'unsigned char SET_DATA[] = {';
	for i := 0 to pred(256 div 2) do begin
		s := s + '0x' + IntToHex(setData[i], 2) + ',';
	end;

	Memo1.Lines.Add(s+'};');
end;

#endif
