
#ifndef __USMTILEMAP_H__
#define __USMTILEMAP_H__

const int MAPDATA_TILE      = 0;
const int MAPDATA_OBJECT    = 1;
const int MAPDATA_ATTRIBUTE = 2;
const int MAPDATA_CHARACTER = 3;

const int MAX_USER_PARAMETER = 10;
const int MAX_USER_INDEX = 256;

typedef union
{
	struct
	{
		unsigned char tile;
		unsigned char object;
		unsigned char attribute;
		unsigned char character;
	} data;
	unsigned long packedData;
} TMapData;

typedef struct
{
	char id[16];
	long wSize;
	long hSize;
	long pitch;
	unsigned char moveableTable[4][256/8];
	unsigned char passableTable[4][256/8];
	unsigned char enablePlane;
	unsigned char defaultTile;
} TMapHeader;

typedef struct
{
	char* szName;
} TTileInfo;

class CTileMap
{
private:
	TMapHeader m_mapHeader;
	TMapData*  m_pMapData;
	TTileInfo  m_tileInfo[2][256];
	int  m_userParameter[MAX_USER_INDEX][MAX_USER_PARAMETER];

	void m_SetMoveableTile(int attribute, int ixTile, bool isMoveable);
	void m_SetPassableTile(int attribute, int ixTile, bool isPassable);

public:
	CTileMap(void);
	~CTileMap(void);

	TMapData* Map(int x, int y);
	int* UserParameter(int index);
	char* TileName(int attr, int index) { return m_tileInfo[attr][index].szName; };

	bool IsMoveableMap(int x, int y);
	bool IsPassableMap(int x, int y);
	int  Width(void) { return m_mapHeader.wSize; };
	int  Height(void) { return m_mapHeader.hSize; };

	bool Create(const char* szId, int width, int heigth);
	bool Destroy(void);
	bool LoadFromText(const char* szTextFile);
	bool LoadFromFile(const char* szMapFile);
	bool LoadFromFile_Lore(const char* szMapName);
	bool LoadFromFile_Dejavu(const char* szMapName);
	bool SaveToFile(const char* szMapFile);
};

#endif // #ifndef __USMTILEMAP_H__
