
#include <assert.h>

#include "pdm_chara.h"
#include "pdm_res.h"
#include "pdm_config.h"

bool CSorakChara::Init(TCharaDesc& charaDesc)
{
	t_CharaDesc = charaDesc;

	if ((g_p_pdm_res) && (g_p_pdm_res->pTileMap))
		g_p_pdm_res->pTileMap->Map(t_CharaDesc.x, t_CharaDesc.y)->data.character = (unsigned char)t_CharaDesc.id;

	return true;
}

bool CSorakChara::Done(void)
{
	return true;
}

bool CSorakChara::GetFace(TBound& bound)
{
	TPoint faceDir;
	this->GetFaceDir(faceDir);

	int face = 0;

	if (faceDir.x == 0)
		face = (faceDir.y < 0) ? 3 : 0;
	else
		face = (faceDir.x < 0) ? 1 : 2;

	face *= 2;
	face += (t_CharaDesc.faceCount % 2);

	bound.x = (face) * TILE_WPITCH;
	bound.y = t_CharaDesc.face * TILE_HPITCH;
	bound.w = TILE_WSIZE;
	bound.h = TILE_HSIZE;

	return true;
}

bool CSorakChara::Action(TCharaMessage message, long param1, long param2)
{
	t_CharaDesc.faceCount += 1;

	switch (message)
	{
	case CHARAMESSAGE_NONE:
		break;

	case CHARAMESSAGE_MOVE:
	case CHARAMESSAGE_PUSH:
		do
		{
			enum TActionType
			{
				ACTION_MOVE,
				ACTION_PUSH,
				ACTION_PULL,
			} actionType = ACTION_MOVE;

			int dx = param1;
			int dy = param2;
			CTileMap* pTileMap = g_p_pdm_res->pTileMap;
			assert(pTileMap);

			if (message == CHARAMESSAGE_MOVE)
			{
				// turn a face
				t_CharaDesc.faceDir = (dx << 16) | (dy & 0xFFFF);
			}
			else
			{
				actionType = ((short(t_CharaDesc.faceDir >> 16) + dx == 0) && (short(t_CharaDesc.faceDir & 0xFFFF) + dy == 0)) ? ACTION_PULL : ACTION_PUSH;
			}

			///////////////////////
			// ó

			// ൿ  ü ִ ˻
			switch (actionType)
			{
			case ACTION_PUSH:
				{
					//    
					if (pTileMap->Map(t_CharaDesc.x + dx, t_CharaDesc.y + dy)->data.character > 0)
						break;

					int onObject = pTileMap->Map(t_CharaDesc.x + dx, t_CharaDesc.y + dy)->data.object;
					// ü    .
					if (onObject == 0)
						break;

					//?? ü б Ӽ ˻ؾ 
					if (false)
						break;

					// ü з  ־ Ѵ
					if (pTileMap->Map(t_CharaDesc.x + 2*dx, t_CharaDesc.y + 2*dy)->data.object > 0)
						break;
					if (!pTileMap->IsMoveableMap(t_CharaDesc.x + 2*dx, t_CharaDesc.y + 2*dy))
						break;

					// ü ̵
					pTileMap->Map(t_CharaDesc.x + 1*dx, t_CharaDesc.y + 1*dy)->data.object = 0;
					pTileMap->Map(t_CharaDesc.x + 2*dx, t_CharaDesc.y + 2*dy)->data.object = onObject;
				}
				break;
			case ACTION_MOVE:
			case ACTION_PULL:
				break;
			default:
				assert(false);
			}

			///////////////////////
			// ĳ ̵

			if (pTileMap->IsMoveableMap(t_CharaDesc.x + dx, t_CharaDesc.y + dy))
			{
				pTileMap->Map(t_CharaDesc.x, t_CharaDesc.y)->data.character = 0;

				t_CharaDesc.x += dx;
				t_CharaDesc.y += dy;

				pTileMap->Map(t_CharaDesc.x, t_CharaDesc.y)->data.character = (unsigned char)t_CharaDesc.id;
			}

			///////////////////////
			// ó

			switch (actionType)
			{
			case ACTION_MOVE:
			case ACTION_PUSH:
				break;
			case ACTION_PULL:
				{
					//    
					if (pTileMap->Map(t_CharaDesc.x - 2*dx, t_CharaDesc.y - 2*dy)->data.character > 0)
						break;

					int onObject = pTileMap->Map(t_CharaDesc.x - 2*dx, t_CharaDesc.y - 2*dy)->data.object;
					// ü    .
					if (onObject == 0)
						break;

					//?? ü  Ӽ ˻ؾ 
					if (false)
						break;

					// ü   ־ Ѵ
					if (pTileMap->Map(t_CharaDesc.x - 1*dx, t_CharaDesc.y - 1*dy)->data.object > 0)
						break;
					if (!pTileMap->IsMoveableMap(t_CharaDesc.x - 1*dx, t_CharaDesc.y - 1*dy))
						break;

					// ü 
					pTileMap->Map(t_CharaDesc.x - 2*dx, t_CharaDesc.y - 2*dy)->data.object = 0;
					pTileMap->Map(t_CharaDesc.x - 1*dx, t_CharaDesc.y - 1*dy)->data.object = onObject;
				}
				break;
			default:
				assert(false);
			}

		} while (0);
		break;

	case CHARAMESSAGE_TALK:
		{
/*??
			extern int CharaData001(char* szKeyWord);
			extern int CharaData002(char* szKeyWord);
			extern int CharaData003(char* szKeyWord);

			typedef int (*FnCharaData)(char* szKeyWord); 
			FnCharaData fnCharaData[] =
			{
				NULL, NULL, CharaData001, CharaData002, CharaData003
			};

			FnCharaData fnTalk;

			if ((this->t_CharaDesc.id <= 1) || (this->t_CharaDesc.id >= sizeof(fnCharaData) / sizeof(FnCharaData)))
				break;

			fnTalk = fnCharaData[this->t_CharaDesc.id];

			if (fnTalk)
			{
				fnTalk((char*)param2);
			}

			pAvejConsole->DisplayText(AVEJ_TEXT(""), 0xFF000000);
*/
		}
		break;

	default:
		// Invalid parameter
		assert(0);
		break;
	}

	return true;
}
