
#ifndef __SORAKCHARA_H__
#define __SORAKCHARA_H__

#include <string.h>

#include "pdm_type.h"
#include "pdm_item.h"

#if !defined(PURE)
	#define PURE  = 0
#endif

typedef enum
{
	CHARAMESSAGE_NONE, // (NULL, NULL)
	CHARAMESSAGE_MOVE, // (dx, dy)
	CHARAMESSAGE_PUSH, // (dx, dy)
	CHARAMESSAGE_TALK, // (NULL, NULL)
} TCharaMessage;

class ISorakChara
{
public:
	enum EEQUIPMENT
	{
		EQUIPMENT_AUXARMOR = 0,
		EQUIPMENT_ARMOR    = 1,
		EQUIPMENT_WEAPON   = 2,
		EQUIPMENT_LEFTHAND = 3,
		EQUIPMENT_AUX1     = 4,
		EQUIPMENT_AUX2     = 5,
		EQUIPMENT_MAX      = 6,
	};

	enum EBACKPACK
	{
		BACKPACK_MAX       = 10,
	};

	typedef struct
	{
		char szName[20];
		long id;
		long x;
		long y;
		long face;
		long type;
		long faceDir;
		long faceCount;
		TSorakItem equipment[EQUIPMENT_MAX];
		TSorakItem backPack[BACKPACK_MAX];
	} TCharaDesc;

	TSorakItem& m_GetNullItem(void)
	{
		static TSorakItem s_item(0);
		return s_item;
	}

public:
	ISorakChara()
		{ memset(&t_CharaDesc, 0, sizeof(t_CharaDesc)); };
	virtual ~ISorakChara()
		{ };

	virtual bool Init(TCharaDesc& charaDesc) PURE;
	virtual bool Done(void) PURE;
	virtual bool GetFace(TBound& bound) PURE;
	virtual bool Action(TCharaMessage message, long param1 = 0, long param2 = 0) PURE;

	long GetX(void)
		{ return t_CharaDesc.x; };
	long GetY(void)
		{ return t_CharaDesc.y; };
	void GetFaceDir(TPoint& dir)
		{
			short dx = short(t_CharaDesc.faceDir>>16);
			short dy = short(t_CharaDesc.faceDir);
			dir.x = dx;
			dir.y = dy;
		};
	TSorakItem& GetEquipment(int index)
		{
			if ((index < 0) || (index >= EQUIPMENT_MAX))
				return m_GetNullItem();
			return t_CharaDesc.equipment[index];
		}
	TSorakItem& GetBackPack(int index)
		{
			if ((index < 0) || (index >= BACKPACK_MAX))
				return m_GetNullItem();
			return t_CharaDesc.backPack[index];
		}

protected:
	TCharaDesc t_CharaDesc;
};

class CSorakChara: public ISorakChara
{
public:
	bool Init(TCharaDesc& charaDesc);
	bool Done(void);
	bool GetFace(TBound& bound);
	bool Action(TCharaMessage message, long param1 = 0, long param2 = 0);
};

#endif // __SORAKCHARA_H__
