
#ifndef __SORAKWINDOW_H__
#define __SORAKWINDOW_H__

#include "pdm_type.h"

#define MAKE_REGION(type, index) ((type) << 5) | ((index) & 0x1F)
#define GET_REGION_TYPE(region)  ((region) >> 5)
#define GET_REGION_INDEX(region) ((region) & 0x1F)

class CSorakWindow
{
public:
	// flag definition
	enum EWindowType
	{
		WINDOW_TYPE_MAP        = 0,
		WINDOW_TYPE_MAP_EDITOR = 1,
	};
	enum EMouseButton
	{
		MOUSE_BUTTON_LEFT   = 1,
		MOUSE_BUTTON_RIGHT  = 2,
		MOUSE_BUTTON_MIDDLE = 3,
	};
	enum EShiftState
	{
		SHIFT_STATE_SHIFT = 0x0001,
		SHIFT_STATE_ALT   = 0x0002,
		SHIFT_STATE_CTRL  = 0x0004,
	};
	enum ERegionType
	{
		NONE      = 0,
		CLIENT    = 1,
		REGION_1  = 2,
		REGION_2  = 3,
		REGION_3  = 4,
		REGION_4  = 5,
		REGION_5  = 6,
	};

private:
	long   m_x, m_y;
	bool   m_enabled;
	TRect  m_clientRect;
	TRect  m_windowRect;

	bool   m_isFocused;
	bool   m_isPressed;
	int    m_xPressed;
	int    m_yPressed;

protected:
	bool m_IsFocused(void) { return m_isFocused; }
	virtual const TRect& m_GetClientRect(void);
	virtual const TRect& m_GetWindowRect(void);
	virtual int  m_InRegion(int x, int y);
	virtual bool m_Display(long xMap, long yMap);

	virtual void m_OnMouseDown(EMouseButton button, int shift, int x, int y);
	virtual void m_OnMouseUp(EMouseButton button, int shift, int x, int y);
	virtual void m_OnMouseMove(int shift, int x, int y);
	virtual void m_OnKeyDown(int shift, int key);
	virtual void m_OnKeyUp(int shift, int key);
	virtual void m_OnKeyChar(char ch);
	virtual void m_OnFocus(bool isFocused);

public:
	// constructor & destructor
	CSorakWindow(long x, long y): m_x(x), m_y(y), m_enabled(true), m_isFocused(false), m_isPressed(false) {};
	virtual ~CSorakWindow(void) {};

	// attribute
	void Show(void) { m_enabled = true; };
	void Hide(void) { m_enabled = false; };
	void SetPos(int x, int y) { m_x = x; m_y = y; };
	void GetPos(int& x, int& y) { x = m_x; y = m_y; };
	const TRect& GetClientRect(void) { return m_GetClientRect(); }
	const TRect& GetWindowRect(void) { return m_GetWindowRect(); }
	int  InRegion(int x, int y)
	{
		return m_InRegion(x, y);
	}

	void Update(long xMap, long yMap)
	{
		if (m_enabled)
			m_Display(xMap, yMap);
	}

	void OnMouseDown(EMouseButton button, int shift, int x, int y)
	{
		m_OnMouseDown(button, shift, x, y);
	}
	void OnMouseUp(EMouseButton button, int shift, int x, int y)
	{
		m_OnMouseUp(button, shift, x, y);
	}
	void OnMouseMove(int shift, int x, int y)
	{
		m_OnMouseMove(shift, x, y);
	}
	void OnKeyDown(int shift, int key)
	{
		m_OnKeyDown(shift, key);
	}
	void OnKeyUp(int shift, int key)
	{
		m_OnKeyUp(shift, key);
	}
	void OnKeyChar(char ch)
	{
		m_OnKeyChar(ch);
	}
	void OnFocus(bool isFocused)
	{
		m_OnFocus(isFocused);
	}

	static CSorakWindow* Create(EWindowType type, long x, long y, void* pParam1 = 0, void* pParam2 = 0);
};

#endif // #ifndef __SORAKWINDOW_H__
