
#ifndef __SORAKWINDOWMAP_H__
#define __SORAKWINDOWMAP_H__

#include "pdm_window.h"
#include "pdm_chara.h"

class CSorakWindowMap: public CSorakWindow
{
private:
	// state machine
	enum TState
	{
		STATE_NONE,
		STATE_MOVE,
		STATE_MENU,
		STATE_CURSOR,
	};

	enum TKeyGroup
	{
		KEYGROUP_NONE,
		KEYGROUP_MOVE,
		KEYGROUP_MENU,
		KEYGROUP_CURSOR,
		KEYGROUP_ENTER,
		KEYGROUP_CANCEL,
	};

	TKeyGroup m_Key2Group(int key);

	void (CSorakWindowMap::*m_fnState)(int shift, int key, TKeyGroup group);

	void m_StateMove(int shift, int key, TKeyGroup group);
	void m_StateCursor(int shift, int key, TKeyGroup group);
	void m_StateMenu(int shift, int key, TKeyGroup group);
	void m_ChangeState(TState type);

	// main character
	void* m_pBaseChara;

	// cursor
	struct TCursor
	{
		enum TIcon
		{
			ICON_LOOK,
			ICON_GET,
			ICON_TALK,
			ICON_ATTACK,
		};
		bool  enable;
		int   limit;
		int   xPos;
		int   yPos;
		TIcon icon;
		TBound iconRect;
		void  SetIcon(TIcon newIcon)
		{
			switch (newIcon)
			{
			case ICON_LOOK:
				iconRect.x = 72;
				iconRect.y = 457;
				iconRect.w = 24;
				iconRect.h = 24;
				break;
			case ICON_GET:
				iconRect.x = 72+24*1;
				iconRect.y = 457;
				iconRect.w = 24;
				iconRect.h = 24;
				break;
			case ICON_TALK:
/*
				iconRect.x = 72-24;
				iconRect.y = 457;
*/
				iconRect.x = 72 - 24*3;
				iconRect.y = 457 - 24*5;
				iconRect.w = 24;
				iconRect.h = 24;
				break;
			case ICON_ATTACK:
				iconRect.x = 72+24*3;
				iconRect.y = 457;
				iconRect.w = 24;
				iconRect.h = 24;
				break;
			}
			icon = newIcon;
		}
	} m_cursor;

	// shift key state
	struct TShiftKey
	{
		enum
		{
			SHIFTKEY_NONE  = 0x00000000,
			SHIFTKEY_LEFT  = 0x00000001,
			SHIFTKEY_RIGHT = 0x00000002,
//			SHIFTKEY_MAX   = (int)0x7FFFFFFF
		};
		int state;
	} m_shiftKey;

protected:
	int   m_InRegion(int x, int y);
	bool  m_Display(long xMap, long yMap);

	void  m_OnMouseDown(EMouseButton button, int shift, int x, int y);
	void  m_OnKeyDown(int shift, int key);
	void  m_OnKeyUp(int shift, int key);
	void  m_OnKeyChar(char ch);

public:
	enum TRegionType
	{
		NONE      = CSorakWindow::NONE,
		CLIENT    = CSorakWindow::CLIENT,
		MAP       = CSorakWindow::REGION_1,
		EQUIPMENT = CSorakWindow::REGION_2,
		BACK_PACK = CSorakWindow::REGION_3,
	};
	typedef CSorakWindow::EWindowType  EWindowType;
	typedef CSorakWindow::EMouseButton EMouseButton;
	typedef CSorakWindow::EShiftState  EShiftState;

	CSorakWindowMap(long x, long y, ISorakChara* pBaseChara = NULL);
	~CSorakWindowMap(void);
};

#endif // #ifndef __SORAKWINDOWMAP_H__
