
////////////////////////////////////////////////////////////////////////////////
// uses

#include "flat_experience.h"
#include <string.h>

#if (TARGET_DEVICE != TARGET_GP2XWIZ)
	#define BIT(number) (1<<(number))

	enum
	{
		BTN_A       = BIT(20),
		BTN_B       = BIT(21),
		BTN_X       = BIT(22),
		BTN_Y       = BIT(23),
		BTN_L       = BIT(7),
		BTN_R       = BIT(6),
		BTN_MENU    = BIT(9),
		BTN_SELECT  = BIT(8),
		BTN_LEFT    = BIT(16),
		BTN_RIGHT   = BIT(17),
		BTN_UP      = BIT(18),
		BTN_DOWN    = BIT(19),
		BTN_VOLUP   = BIT(10),
		BTN_VOLDOWN = BIT(11)
	};

	struct tsdev { int dummy; };

	typedef unsigned long  uint32_t;

	extern uint32_t lc_getbuttons(void);

 #if defined(_MSC_VER)
	#include <windows.h>
	#pragma comment(lib, "winmm.lib")
 #endif

#else
	extern "C"
	{
		#include "castor.h"
	}
	#include <stdlib.h>
	#include <tslib.h>
#endif


////////////////////////////////////////////////////////////////////////////////
// Input device

static struct
{
	int  wiz_key;
	int  common_key;
} KEY_MAPPING_TABLE[] =
{
	{ BTN_A,       flat_experience::KEY_A       },
	{ BTN_B,       flat_experience::KEY_B       },
	{ BTN_X,       flat_experience::KEY_X       },
	{ BTN_Y,       flat_experience::KEY_Y       },
	{ BTN_L,       flat_experience::KEY_L       },
	{ BTN_R,       flat_experience::KEY_R       },
	{ BTN_MENU,    flat_experience::KEY_MENU    },
	{ BTN_SELECT,  flat_experience::KEY_SELECT  },
	{ BTN_LEFT,    flat_experience::KEY_LEFT    },
	{ BTN_RIGHT,   flat_experience::KEY_RIGHT   },
	{ BTN_UP,      flat_experience::KEY_UP      },
	{ BTN_DOWN,    flat_experience::KEY_DOWN    },
	{ BTN_VOLUP,   flat_experience::KEY_VOLUP   },
	{ BTN_VOLDOWN, flat_experience::KEY_VOLDOWN }
};

class CInputDevice: public flat_experience::IInputDevice
{
public:
	CInputDevice()
	:	m_once_pressed(0), m_ts_device(NULL), m_ts_device_fd(-1)
	{
		memset(&m_input_update_info, 0, sizeof(m_input_update_info));

	#if (TARGET_DEVICE != TARGET_GP2XWIZ)
	#else
		{
			const char* ts_device_name = getenv("SDL_MOUSEDEV");

			if (ts_device_name == NULL)
				ts_device_name = getenv("TSLIB_TSDEVICE");

			if (ts_device_name)
			{
				m_ts_device   = ts_open(ts_device_name, 1);
				m_ts_device_fd = ((m_ts_device) && (ts_config(m_ts_device) >= 0)) ? ts_fd(m_ts_device) : -1;
			}
		}
	#endif
	}
	virtual ~CInputDevice()
	{
	#if (TARGET_DEVICE != TARGET_GP2XWIZ)
	#else
		if (m_ts_device)
			ts_close(m_ts_device);
	#endif
	}
	virtual const flat_experience::TInputUpdateInfo& Update(void)
	{
		unsigned long wiz_key_flag = lc_getbuttons();
		m_input_update_info.key_down_flag = 0;
		for (unsigned int i = 0; i < sizeof(KEY_MAPPING_TABLE) / sizeof(KEY_MAPPING_TABLE[0]); i++)
		{
			if (wiz_key_flag & KEY_MAPPING_TABLE[i].wiz_key)
			{
				m_input_update_info.key_down_flag |= KEY_MAPPING_TABLE[i].common_key;

				if (!(m_once_pressed & KEY_MAPPING_TABLE[i].common_key))
					m_input_update_info.key_pressed_flag |= KEY_MAPPING_TABLE[i].common_key;

				m_once_pressed |= KEY_MAPPING_TABLE[i].common_key;
			}
			else
			{
				m_once_pressed &= ~KEY_MAPPING_TABLE[i].common_key;
			}
		}

	#if defined(_MSC_VER)
		// ġ
		if ((m_input_update_info.is_touched = (HIBYTE(GetAsyncKeyState(VK_LBUTTON)) > 0)))
		{
			extern void GetCursorPosLocal(int& x, int& y);

			GetCursorPosLocal(m_input_update_info.touch_pos.x, m_input_update_info.touch_pos.y);
		}
		// ð
		{
			m_input_update_info.time_stamp = timeGetTime();
		}
	#elif (TARGET_DEVICE == TARGET_GP2XWIZ)
		// ġ
		{
			m_input_update_info.is_touched = false;

			ts_sample touch_state;
			{
				while (ts_read(m_ts_device, &touch_state, 1) > 0)
				{
					m_input_update_info.is_touched  = (touch_state.pressure > 0);
					m_input_update_info.touch_pos.x = touch_state.x;
					m_input_update_info.touch_pos.y = touch_state.y;
				}
			}
		}
		// ð
		{
			static bool    s_is_first = true;
			static timeval s_start_tick;

			if (s_is_first)
			{
				gettimeofday(&s_start_tick, NULL);
				s_is_first = false;
			}

			timeval current_tick;
			gettimeofday(&current_tick, NULL);

			m_input_update_info.time_stamp = (current_tick.tv_sec - s_start_tick.tv_sec) * 1000 + (current_tick.tv_usec - s_start_tick.tv_usec) / 1000;
		}
	#elif defined(_TARGET_BADA)
		//??
	#else
		//??
	#endif

		return m_input_update_info;
	}
	virtual const flat_experience::TInputUpdateInfo& GetUpdateInfo(void) const
	{
		return m_input_update_info;
	}
	virtual bool CheckKeyDown(flat_experience::TKey key)
	{
		return (m_input_update_info.key_down_flag & int(key)) != 0;
	}
	virtual bool CheckKeyPressed(flat_experience::TKey key)
	{
		if ((m_input_update_info.key_pressed_flag & int(key)) == 0)
			return false;

		m_input_update_info.key_pressed_flag &= ~int(key);

		return true;
	}

private:
	flat_experience::TInputUpdateInfo  m_input_update_info;
	unsigned long                      m_once_pressed;
	tsdev*                             m_ts_device;
	int                                m_ts_device_fd;

};

class CInputDeviceHolder: public flat_experience::IInputDevice
{
public:
	CInputDeviceHolder(flat_experience::IInputDevice* p_input_device)
	:	m_p_input_device(p_input_device)
	{
		memset(&m_update_info_null, 0, sizeof(m_update_info_null));
	}
	virtual ~CInputDeviceHolder()
	{
	}
	virtual  const flat_experience::TInputUpdateInfo& Update(void)
	{
		return (m_p_input_device) ? m_p_input_device->Update() : m_update_info_null;
	}
	virtual  const flat_experience::TInputUpdateInfo& GetUpdateInfo(void) const
	{
		return (m_p_input_device) ? m_p_input_device->GetUpdateInfo() : m_update_info_null;
	}
	virtual  bool  CheckKeyDown(flat_experience::TKey key)
	{
		return (m_p_input_device) ? m_p_input_device->CheckKeyDown(key) : false;
	}
	virtual  bool  CheckKeyPressed(flat_experience::TKey key)
	{
		return (m_p_input_device) ? m_p_input_device->CheckKeyPressed(key) : false;
	}

private:
	flat_experience::IInputDevice*    m_p_input_device;
	flat_experience::TInputUpdateInfo m_update_info_null;
};

////////////////////////////////////////////////////////////////////////////////
// namespace flat_experience

namespace flat_experience
{
	static CInputDevice s_input_device;

	IInputDevice* GetInputDevice(void)
	{
		return new CInputDeviceHolder(&s_input_device);
	}
}
