
#include "sm_util.h"
#include "image_decoder_png.h"

#include <memory>
#include <cassert>


////////////////////////////////////////////////////////////////////////////////
// definition

#if defined(_TARGET_BADA)
#	define _AVEJ_SUPPORT_FSTREAM 0
#else
#	define _AVEJ_SUPPORT_FSTREAM 1
#endif

////////////////////////////////////////////////////////////////////////////////
// image decoder customizing

static flat_experience::ICanvas* s_CreateCanvasFromPNG(TPixelFormat pixel_format, const unsigned char* p_stream, unsigned int stream_size, bool use_transparent_color = false, unsigned long transparent_color = 0)
{
	image_decoder::png::TImage png_image;

	if (!image_decoder::png::Decode(p_stream, stream_size, png_image))
		return 0;

	flat_experience::ICanvas* p_ret_bitmap = 0;

	switch (pixel_format)
	{
	case PIXELFORMAT_ARGB8888:
		{
			typedef unsigned long TDestPixel;

			std::auto_ptr<TDestPixel> buffer(new TDestPixel[png_image.width * png_image.height]);

			switch (png_image.depth)
			{
			case 24:
				{
					for (int y = 0; y < png_image.height; y++)
					{
						TDestPixel*    p_dest = buffer.get() + png_image.width * y;
						unsigned char* p_sour = ((unsigned char**)png_image.p_bitmap)[y];
						for (int x = 0; x < png_image.width; x++)
						{
							*p_dest  = 0xFF000000;
							*p_dest |= ((TDestPixel(*p_sour++)) << 16);
							*p_dest |= ((TDestPixel(*p_sour++)) <<  8);
							*p_dest |= ((TDestPixel(*p_sour++)) <<  0);
							++p_dest;
						}
					}

					p_ret_bitmap = (use_transparent_color) ?
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get(), transparent_color) :
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get());
				}
				break;
			case 32:
				{
					for (int y = 0; y < png_image.height; y++)
					{
						TDestPixel*    p_dest = buffer.get() + png_image.width * y;
						unsigned char* p_sour = ((unsigned char**)png_image.p_bitmap)[y];
						for (int x = 0; x < png_image.width; x++)
						{
							*p_dest  = ((TDestPixel(*p_sour++)) << 16);
							*p_dest |= ((TDestPixel(*p_sour++)) <<  8);
							*p_dest |= ((TDestPixel(*p_sour++)) <<  0);
							*p_dest |= ((TDestPixel(*p_sour++)) << 24);
							++p_dest;
						}
					}

					p_ret_bitmap = (use_transparent_color) ?
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get(), transparent_color) :
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get());
				}
				break;
			default:
				return 0;
			}
		}
		break;
	case PIXELFORMAT_RGB565:
		{
			typedef unsigned short TDestPixel;

			if (use_transparent_color)
			{
				unsigned long r = (transparent_color >> 19) & 0x1F;
				unsigned long g = (transparent_color >> 10) & 0x3F;
				unsigned long b = (transparent_color >>  3) & 0x1F;

				transparent_color = (r << 11) | (g << 5) | (b);
			}

			std::auto_ptr<TDestPixel> buffer(new TDestPixel[png_image.width * png_image.height]);

			switch (png_image.depth)
			{
			case  1:
				{
					assert(false);
				}
				break;
			case  4:
				{
					assert(false);
				}
				break;
			case  8:
				{
					TDestPixel target_palette[256];

					for (int i = 0; i < 256; i++)
					{
						target_palette[i]  = ((TDestPixel((png_image.palette[i] >> 19) & 0x1F)) << 11) & 0xF800;
						target_palette[i] |= ((TDestPixel((png_image.palette[i] >> 10) & 0x3F)) <<  5) & 0x07E0;
						target_palette[i] |= ((TDestPixel((png_image.palette[i] >>  3) & 0x1F)));
					}

					for (int y = 0; y < png_image.height; y++)
					{
						TDestPixel*    p_dest = buffer.get() + png_image.width * y;
						unsigned char* p_sour = ((unsigned char**)png_image.p_bitmap)[y];
						for (int x = 0; x < png_image.width; x++)
						{
//							unsigned int index = (x % 2) ? (*p_sour++ & 0x0F) : ((*p_sour >> 4) & 0x0F);
							*p_dest++ = target_palette[*p_sour++];
						}
					}

					p_ret_bitmap = (use_transparent_color) ?
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get(), transparent_color) :
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get());
				}
				break;
			case 24:
				{
					for (int y = 0; y < png_image.height; y++)
					{
						TDestPixel*    p_dest = buffer.get() + png_image.width * y;
						unsigned char* p_sour = ((unsigned char**)png_image.p_bitmap)[y];
						for (int x = 0; x < png_image.width; x++)
						{
							*p_dest  = ((TDestPixel(*p_sour++)) << 8) & 0xF800;
							*p_dest |= ((TDestPixel(*p_sour++)) << 3) & 0x07E0;
							*p_dest |= ((TDestPixel(*p_sour++)) >> 3);
							++p_dest;
						}
					}

					p_ret_bitmap = (use_transparent_color) ?
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get(), transparent_color) :
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get());
				}
				break;
			case 32:
				{
					for (int y = 0; y < png_image.height; y++)
					{
						TDestPixel*    p_dest = buffer.get() + png_image.width * y;
						unsigned char* p_sour = ((unsigned char**)png_image.p_bitmap)[y];
						for (int x = 0; x < png_image.width; x++)
						{
							*p_dest  = ((TDestPixel(*p_sour++)) << 8) & 0xF800;
							*p_dest |= ((TDestPixel(*p_sour++)) << 3) & 0x07E0;
							*p_dest |= ((TDestPixel(*p_sour++)) >> 3);
							++p_sour;
							++p_dest;
						}
					}

					p_ret_bitmap = (use_transparent_color) ?
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get(), transparent_color):
						flat_experience::CreateCanvas(pixel_format, png_image.width, png_image.height, buffer.get());
				}
				break;
			default:
				return 0;
			}
		}
		break;
	default:
		break;
	}

	delete[] png_image.p_bitmap;

	return p_ret_bitmap;
}


#if (_AVEJ_SUPPORT_FSTREAM)
	#include <fstream>
	static flat_experience::ICanvas* s_CreateCanvasFromPNG(TPixelFormat pixel_format, const char* sz_file_name, bool use_transparent_color = false, unsigned long transparent_color = 0)
	{
		std::ifstream stream(sz_file_name, std::ios::binary | std::ios::in);
		
		if (!stream)
			return 0;
	
		stream.seekg(0, std::ios_base::end);
		unsigned int file_length = stream.tellg();
		stream.seekg(0, std::ios_base::beg);
		
		std::auto_ptr<unsigned char> buffer(new unsigned char[file_length]);
		stream.read((char*)buffer.get(), file_length);
	
		return s_CreateCanvasFromPNG(pixel_format, buffer.get(), file_length, use_transparent_color, transparent_color);
	}
#else
	#if defined(_TARGET_BADA)

		#include <FIo.h>
		
		static flat_experience::ICanvas* s_CreateCanvasFromPNG(TPixelFormat pixel_format, const char* sz_file_name, bool use_transparent_color = false, unsigned long transparent_color = 0)
		{
			result r;
			Osp::Io::File file;
			Osp::Base::String fileName(sz_file_name);
		
			fileName = "/Res/" + fileName;
		
			r = file.Construct(fileName, L"r");
		
			if (r != E_SUCCESS)
			{
				AppLog("[FE] File not found (%s)", sz_file_name);
				return 0;
			}
		
			file.Seek(Osp::Io::FILESEEKPOSITION_END, 0);
			int file_length = file.Tell();
			file.Seek(Osp::Io::FILESEEKPOSITION_BEGIN, 0);
		
			std::auto_ptr<unsigned char> buffer(new unsigned char[file_length]);
			file.Read((void*)buffer.get(), file_length);
		
			return s_CreateCanvasFromPNG(pixel_format, buffer.get(), file_length, use_transparent_color, transparent_color);
		}
	#else
		#error "unknown target name"
	#endif
#endif



flat_experience::ICanvas* sm_util::CreateCanvasFromPNG(TPixelFormat pixel_format, const unsigned char* p_stream, unsigned int stream_size)
{
	return s_CreateCanvasFromPNG(pixel_format, p_stream, stream_size);
}

flat_experience::ICanvas* sm_util::CreateCanvasFromPNG(TPixelFormat pixel_format, const unsigned char* p_stream, unsigned int stream_size, unsigned long transparent_color)
{
	return s_CreateCanvasFromPNG(pixel_format, p_stream, stream_size, true, transparent_color);
}

flat_experience::ICanvas* sm_util::CreateCanvasFromPNG(TPixelFormat pixel_format, const char* sz_file_name)
{
	return s_CreateCanvasFromPNG(pixel_format, sz_file_name);
}

flat_experience::ICanvas* sm_util::CreateCanvasFromPNG(TPixelFormat pixel_format, const char* sz_file_name, unsigned long transparent_color)
{
	return s_CreateCanvasFromPNG(pixel_format, sz_file_name, true, transparent_color);
}
