
#include "flat_experience.h"

#if !defined(_TARGET_BADA)

#include <stdio.h>

struct flat_experience::file_io::CStreamReadFile::TImpl
{
	TImpl()
		: p_file(0)
	{
	}
	FILE* p_file;
};

#define IS_AVAILABLE  (p_impl != 0)

flat_experience::file_io::CStreamReadFile::CStreamReadFile(const char* sz_file_name)
{
	p_impl = new TImpl;

	if (p_impl)
	{
		p_impl->p_file = fopen(sz_file_name, "rb");
		if (p_impl->p_file == 0)
		{
			delete p_impl;
			p_impl = 0;
		}
	}
}

flat_experience::file_io::CStreamReadFile::~CStreamReadFile(void)
{
	if (IS_AVAILABLE)
		fclose(p_impl->p_file);
}

long flat_experience::file_io::CStreamReadFile::Read(void* p_buffer, long count)
{
	if (!IS_AVAILABLE)
		return 0;

	return fread(p_buffer, 1, count, p_impl->p_file);
}

long flat_experience::file_io::CStreamReadFile::Seek(TSeek origin, long offset)
{
	if (!IS_AVAILABLE)
		return -1;

	switch (origin)
	{
	case SEEK_BEGIN:
		fseek(p_impl->p_file, offset, SEEK_SET);
		break;
	case SEEK_CURRENT:
		fseek(p_impl->p_file, offset, SEEK_CUR);
		break;
	case SEEK_END:
		fseek(p_impl->p_file, offset, SEEK_END);
		break;
	default:
		return -1;
	}

	return ftell(p_impl->p_file);
}

long flat_experience::file_io::CStreamReadFile::GetSize(void)
{
	if (!IS_AVAILABLE)
		return -1;

	long  result;
	long  current_pos = ftell(p_impl->p_file);

	fseek(p_impl->p_file, 0, SEEK_END);
	result = ftell(p_impl->p_file);

	fseek(p_impl->p_file, current_pos, SEEK_SET);

	return result;
}

bool flat_experience::file_io::CStreamReadFile::IsValidPos(void)
{
	if (!IS_AVAILABLE)
		return false;

	return (feof(p_impl->p_file) == 0);
}

#else // #if !defined(_TARGET_BADA)

#include <FIo.h>

struct flat_experience::file_io::CStreamReadFile::TImpl
{
	TImpl()
		: file_size(0)
	{
	}
	Osp::Io::File file;
	int           file_size;
};

#define IS_AVAILABLE  (p_impl != 0)

flat_experience::file_io::CStreamReadFile::CStreamReadFile(const char* sz_file_name)
{
	p_impl = new TImpl;

	if (p_impl)
	{
		try
		{
			result r;
			Osp::Base::String fileName(sz_file_name);

			fileName = "/Res/" + fileName;

			r = p_impl->file.Construct(fileName, L"r");

			if (r == E_SUCCESS)
			{
				p_impl->file_size = this->GetSize();
			}
			else
			{
				AppLog("[FE] File not found (%s)", sz_file_name);

				throw 1;
			}
		}
		catch (...)
		{
			delete p_impl;
			p_impl = 0;
		}
	}
}

flat_experience::file_io::CStreamReadFile::~CStreamReadFile(void)
{
}

long flat_experience::file_io::CStreamReadFile::Read(void* p_buffer, long count)
{
	if (!IS_AVAILABLE)
		return 0;

	return p_impl->file.Read(p_buffer, count);
}

long flat_experience::file_io::CStreamReadFile::Seek(TSeek origin, long offset)
{
	if (!IS_AVAILABLE)
		return -1;

	switch (origin)
	{
	case Osp::Io::FILESEEKPOSITION_BEGIN:
		p_impl->file.Seek(Osp::Io::FileSeekPosition(origin), offset);
		break;
	case Osp::Io::FILESEEKPOSITION_CURRENT:
		p_impl->file.Seek(Osp::Io::FileSeekPosition(origin), offset);
		break;
	case Osp::Io::FILESEEKPOSITION_END:
		p_impl->file.Seek(Osp::Io::FileSeekPosition(origin), offset);
		break;
	default:
		return -1;
	}

	return p_impl->file.Tell();
}

long flat_experience::file_io::CStreamReadFile::GetSize(void)
{
	if (!IS_AVAILABLE)
		return -1;

	long  res;
	long  current_pos = p_impl->file.Tell();

	p_impl->file.Seek(Osp::Io::FILESEEKPOSITION_END, 0);
	res = p_impl->file.Tell();

	p_impl->file.Seek(Osp::Io::FILESEEKPOSITION_BEGIN, current_pos);

	return res;
}

bool flat_experience::file_io::CStreamReadFile::IsValidPos(void)
{
	if (!IS_AVAILABLE)
		return false;

	return (p_impl->file.Tell() < p_impl->file_size);
}

#endif // #if !defined(_TARGET_BADA)
